#include <string.h>
#include <dos.h>
#include "gravultr.h"


typedef void (far *ultramid_hook)(void);
static ultramid_hook ultramid_TSR_entrypoint = NULL;

const ULTRAMID_SET_PAN            = 2;
const ULTRAMID_LOAD_PATCH         = 12;
const ULTRAMID_UNLOAD_PATCH       = 13;
const ULTRAMID_UNLOAD_ALL_PATCHES = 15;
const ULTRAMID_MIDI_OUT           = 16;
const ULTRAMID_ALL_NOTES_OFF      = 18;

/*************************************************************************
  Looks for the interrupt vector that the ultramid TSR hooked itself
  into. If found, it remembers the address internally, and returns 0
  for success. If not found, returns -1.
 ************************************************************************/
int gus_set_ultrasound_hook (void)
{
   int vect;

   for (vect = 0x78; vect <= 0x7f; vect++)
   {
      char far *recog_text;

      ultramid_TSR_entrypoint = (ultramid_hook)getvect(vect);
      recog_text = (char far *)
         MK_FP(FP_SEG(ultramid_TSR_entrypoint), 0x103);

      if (_fstrncmp(recog_text, "ULTRAMID", 8) == 0)
         return 0;
   }
   // TSR not found
   ultramid_TSR_entrypoint = NULL;
   return -1;
}


/*************************************************************************
   Sends a midi byte to the Gravis Ultrasound synthesizer.
 ************************************************************************/
void gus_send_midibyte (int midibyte)
{
   if (!ultramid_TSR_entrypoint)
      return;

   _AX = ULTRAMID_MIDI_OUT;
   _CX = midibyte;
   (*ultramid_TSR_entrypoint)();
}


/*************************************************************************
   Loads sample data for the specified patch (or Midi program
   change) into the card's internal RAM. It does not perform
   the actual Midi program change, but it's ready for it.
 ************************************************************************/
void gus_load_patch (int patch_number)
{
   if (!ultramid_TSR_entrypoint)
      return;

   _AX = ULTRAMID_LOAD_PATCH;
   _CX = patch_number;
   (*ultramid_TSR_entrypoint)();
}

/*************************************************************************
   Unloads sample data for the specified patch (or Midi program
   change) from the card's internal RAM.
 ************************************************************************/
void gus_unload_patch (int patch_number)
{
   if (!ultramid_TSR_entrypoint)
      return;

   _AX = ULTRAMID_UNLOAD_PATCH;
   _CX = patch_number;
   (*ultramid_TSR_entrypoint)();
}


/*************************************************************************

 ************************************************************************/
void gus_unload_all_patches (void)
{
   if (!ultramid_TSR_entrypoint)
      return;

   _AX = ULTRAMID_UNLOAD_ALL_PATCHES;
   (*ultramid_TSR_entrypoint)();
}


/*************************************************************************

 ************************************************************************/
void gus_all_midi_voices_off (void)
{
   if (!ultramid_TSR_entrypoint)
      return;

   _AX = ULTRAMID_ALL_NOTES_OFF;
   (*ultramid_TSR_entrypoint)();
}

/*************************************************************************

 ************************************************************************/
void gus_set_pan (int voice, int pan)
{
   if (!ultramid_TSR_entrypoint)
      return;

   _AX = ULTRAMID_SET_PAN;
   _BX = pan;
   _CX = voice;
   (*ultramid_TSR_entrypoint)();
}


#ifdef MAIN
/*************************************************************************
   Test driver.
 ************************************************************************/
#include <stdio.h>
#include <conio.h>

int main (int argc, char **argv)
{
   int n;
   int p;
   int ch;

   if (set_ultrasound_hook())
   {
      printf("ULTRAMID TSR must be loaded first\n");
      return 1;
   }


   load_patch(42);
   for (ch = 0; ch < 16; ch++)
   {
      printf("Channel %d\n", ch);
      send_midibyte(0xC0 + ch);
      send_midibyte(42);

      send_midibyte(0x90 + ch);
      send_midibyte(48);
      send_midibyte(0x40);
      delay(200);
      send_midibyte(0x90 + ch);
      send_midibyte(48);
      send_midibyte(0x00);

      send_midibyte(0x90 + ch);
      send_midibyte(51);
      send_midibyte(0x40);
      delay(200);
      send_midibyte(0x90 + ch);
      send_midibyte(51);
      send_midibyte(0x00);
   }

   for (n = 30; n < 66; n++)
   {
      printf("Note %d\n", n);
      send_midibyte(0x90 + 9);
      send_midibyte(n);
      send_midibyte(0x40);
      delay(400);
   }

   for (p = 0; p < 128; p++)
   {
      printf("Instrument %3d\n", p);
      load_patch(p);
      send_midibyte(0xC0);
      send_midibyte(p);

      for (n = 60; n < 70; n++, n++)
      {
         send_midibyte(0x90);
         send_midibyte(n);
         send_midibyte(0x40);
         delay(150);
         send_midibyte(0x90);
         send_midibyte(n);
         send_midibyte(0x00);
      }
      for (n = 48; n < 58; n++, n++)
      {
         send_midibyte(0x90);
         send_midibyte(n);
         send_midibyte(0x40);
         delay(150);
         send_midibyte(0x90);
         send_midibyte(n);
         send_midibyte(0x00);
      }
      // (void)getch();
      unload_patch(p);
   }

   return 0;
}
#endif